@@ -1,8 +1,40 @@
-use super::NodeId;
+use std::fmt::Display;
+
+use super::{LaneNode, NodeId};
 use thiserror::Error;
 
 #[derive(Debug, Clone, Error)]
-pub enum CompilationError {
+pub struct CompilationError {
+    pub payload: CompilationErrorPayload,
+    /// (lane, card)
+    pub loc: Option<(LaneNode, i32)>,
+}
+
+impl CompilationError {
+    pub fn with_loc(payload: CompilationErrorPayload, lane_id: LaneNode, card_id: i32) -> Self {
+        Self {
+            payload,
+            loc: Some((lane_id, card_id)),
+        }
+    }
+}
+
+impl Display for CompilationError {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        if let Some(loc) = self.loc.as_ref() {
+            write!(
+                f,
+                "CompilationError: [Lane: {} Card: {}], Error: {}",
+                loc.0, loc.1, self.payload
+            )
+        } else {
+            write!(f, "{}", self.payload)
+        }
+    }
+}
+
+#[derive(Debug, Clone, Error)]
+pub enum CompilationErrorPayload {
     #[error("The requested functionality ({0}) is not yet implemented")]
     Unimplemented(&'static str),
 
